@@ -17,6 +17,11 @@ class Adaptive(object):
     Adaptively allocate workers based on scheduler load.  A superclass.
 
     Contains logic to dynamically resize a Dask cluster based on current use.
+    This class needs to be paired with a system that can create and destroy
+    Dask workers using a cluster resource manager.  Typically it is built into
+    already existing solutions, rather than used directly by users.
+    It is most commonly used from the ``.adapt(...)`` method of various Dask
+    cluster classes.
 
     Parameters
     ----------
@@ -36,6 +41,9 @@ class Adaptive(object):
     target_duration: timedelta or str, default "5s"
         Amount of time we want a computation to take.
         This affects how aggressively we scale up.
+    worker_key: Callable[WorkerState]
+        Function to group workers together when scaling down
+        See Scheduler.workers_to_close for more information
     minimum: int
         Minimum number of workers to keep around
     maximum: int
@@ -45,12 +53,26 @@ class Adaptive(object):
 
     Examples
     --------
-    >>> class MyCluster(object):
+
+    This is commonly used from existing Dask classes, like KubeCluster
+
+    >>> from dask_kubernetes import KubeCluster
+    >>> cluster = KubeCluster()
+    >>> cluster.adapt(minimum=10, maximum=100)
+
+    Alternatively you can use it from your own Cluster class by subclassing
+    from Dask's Cluster superclass
+
+    >>> from distributed.deploy import Cluster
+    >>> class MyCluster(Cluster):
     ...     def scale_up(self, n):
     ...         """ Bring worker count up to n """
     ...     def scale_down(self, workers):
     ...        """ Remove worker addresses from cluster """
 
+    >>> cluster = MyCluster()
+    >>> cluster.adapt(minimum=10, maximum=100)
+
     Notes
     -----
     Subclasses can override :meth:`Adaptive.should_scale_up` and
@@ -65,8 +87,9 @@ class Adaptive(object):
 
     def __init__(self, scheduler, cluster, interval='1s', startup_cost='1s',
                  scale_factor=2, minimum=0, maximum=None, wait_count=3,
-                 target_duration='5s', **kwargs):
+                 target_duration='5s', worker_key=lambda x: x, **kwargs):
         interval = parse_timedelta(interval, default='ms')
+        self.worker_key = worker_key
         self.scheduler = scheduler
         self.cluster = cluster
         self.startup_cost = parse_timedelta(startup_cost, default='s')
@@ -204,7 +227,8 @@ class Adaptive(object):
     @gen.coroutine
     def _retire_workers(self, workers=None):
         if workers is None:
-            workers = self.workers_to_close()
+            workers = self.workers_to_close(key=self.worker_key,
+                                            minimum=self.minimum)
         if not workers:
             raise gen.Return(workers)
         with log_errors():
@@ -244,6 +268,7 @@ class Adaptive(object):
         if self.maximum:
             instances = min(self.maximum, instances)
 
+        instances = int(instances)
         logger.info("Scaling up to %d workers", instances)
         return {'n': instances}
 
@@ -255,7 +280,8 @@ class Adaptive(object):
         self._adapting = True
         try:
             should_scale_up = self.should_scale_up()
-            workers = set(self.workers_to_close())
+            workers = set(self.workers_to_close(key=self.worker_key,
+                                                minimum=self.minimum))
             if should_scale_up and workers:
                 logger.info("Attempting to scale up and scale down simultaneously.")
                 return
