@@ -9,12 +9,12 @@ import shutil
 import threading
 import uuid
 
+import dask
 from tornado import gen
 from tornado.ioloop import IOLoop, TimeoutError
 from tornado.locks import Event
 
 from .comm import get_address_host, get_local_address_for, unparse_host_port
-from .config import config
 from .core import rpc, RPCClosed, CommClosedError, coerce_to_address
 from .metrics import time
 from .node import ServerNode
@@ -22,7 +22,7 @@ from .process import AsyncProcess
 from .proctitle import enable_proctitle_on_children
 from .security import Security
 from .utils import (get_ip, mp_context, silence_logging, json_load_robust,
-        PeriodicCallback, parse_timedelta)
+        PeriodicCallback)
 from .worker import _ncores, run, parse_memory_limit
 
 
@@ -48,8 +48,8 @@ class Nanny(ServerNode):
         if scheduler_file:
             cfg = json_load_robust(scheduler_file)
             self.scheduler_addr = cfg['address']
-        elif scheduler_ip is None and config.get('scheduler-address'):
-            self.scheduler_addr = config['scheduler-address']
+        elif scheduler_ip is None and dask.config.get('scheduler-address'):
+            self.scheduler_addr = dask.config.get('scheduler-address')
         elif scheduler_port is None:
             self.scheduler_addr = coerce_to_address(scheduler_ip)
         else:
@@ -64,7 +64,7 @@ class Nanny(ServerNode):
         self.preload_argv = preload_argv
 
         self.contact_address = contact_address
-        self.memory_terminate_fraction = config.get('worker-memory-terminate', 0.95)
+        self.memory_terminate_fraction = dask.config.get('distributed.worker.memory.terminate')
 
         self.security = security or Security()
         assert isinstance(self.security, Security)
@@ -252,15 +252,18 @@ class Nanny(ServerNode):
         """ Track worker's memory.  Restart if it goes above 95% """
         if self.status != 'running':
             return
+        process = self.process.process
+        if process is None:
+            return
         try:
-            proc = psutil.Process(self.process.pid)
+            proc = psutil.Process(process.pid)
         except psutil.NoSuchProcess:
             return
         memory = proc.memory_info().rss
         frac = memory / self.memory_limit
         if self.memory_terminate_fraction and frac > self.memory_terminate_fraction:
             logger.warning("Worker exceeded 95% memory budget.  Restarting")
-            self.process.process.terminate()
+            process.terminate()
 
     def is_alive(self):
         return self.process is not None and self.process.status == 'running'
@@ -341,50 +344,34 @@ class WorkerProcess(object):
             yield self.running.wait()
             raise gen.Return(self.status)
 
-        while True:
-            # FIXME: this sometimes stalls in _wait_until_connected
-            # our temporary solution is to retry a few times if the process
-            # doesn't start up in five seconds
-            self.init_result_q = init_q = mp_context.Queue()
-            self.child_stop_q = mp_context.Queue()
-            uid = uuid.uuid4().hex
-            try:
-                self.process = AsyncProcess(
-                    target=self._run,
-                    kwargs=dict(worker_args=self.worker_args,
-                                worker_kwargs=self.worker_kwargs,
-                                worker_start_args=self.worker_start_args,
-                                silence_logs=self.silence_logs,
-                                init_result_q=self.init_result_q,
-                                child_stop_q=self.child_stop_q,
-                                uid=uid),
-                )
-                self.process.daemon = True
-                self.process.set_exit_callback(self._on_exit)
-                self.running = Event()
-                self.stopped = Event()
-                self.status = 'starting'
-                yield self.process.start()
-                if self.status == 'starting':
-                    timeout = parse_timedelta(config.get('nanny-start-timeout', '30s'))
-                    yield gen.with_timeout(timedelta(seconds=timeout),
-                                           self._wait_until_started(uid))
-            except gen.TimeoutError:
-                logger.info("Failed to start worker process.  Restarting")
-                yield gen.with_timeout(timedelta(seconds=1),
-                                       self.process.terminate())
-            else:
-                break
+        self.init_result_q = init_q = mp_context.Queue()
+        self.child_stop_q = mp_context.Queue()
+        uid = uuid.uuid4().hex
 
-        if self.status == 'starting':
-            msg = yield self._wait_until_connected(uid)
-            if not msg:
-                raise gen.Return(self.status)
-            self.worker_address = msg['address']
-            self.worker_dir = msg['dir']
-            assert self.worker_address
-            self.status = 'running'
-            self.running.set()
+        self.process = AsyncProcess(
+            target=self._run,
+            kwargs=dict(worker_args=self.worker_args,
+                        worker_kwargs=self.worker_kwargs,
+                        worker_start_args=self.worker_start_args,
+                        silence_logs=self.silence_logs,
+                        init_result_q=self.init_result_q,
+                        child_stop_q=self.child_stop_q,
+                        uid=uid),
+        )
+        self.process.daemon = True
+        self.process.set_exit_callback(self._on_exit)
+        self.running = Event()
+        self.stopped = Event()
+        self.status = 'starting'
+        yield self.process.start()
+        msg = yield self._wait_until_connected(uid)
+        if not msg:
+            raise gen.Return(self.status)
+        self.worker_address = msg['address']
+        self.worker_dir = msg['dir']
+        assert self.worker_address
+        self.status = 'running'
+        self.running.set()
 
         init_q.close()
 
@@ -472,25 +459,6 @@ class WorkerProcess(object):
             except Exception as e:
                 logger.error("Failed to kill worker process: %s", e)
 
-    @gen.coroutine
-    def _wait_until_started(self, uid):
-        delay = 0.05
-        while True:
-            if self.status != 'starting':
-                return
-            try:
-                msg = self.init_result_q.get_nowait()
-                if msg['uid'] != uid:  # ensure that we didn't cross queues
-                    continue
-                if msg['status'] != 'started':
-                    logger.warn("Nanny got unexpected message %s. "
-                                "Starting worker again", msg)
-                    raise gen.TimeoutError()
-                return
-            except Empty:
-                yield gen.sleep(delay)
-                continue
-
     @gen.coroutine
     def _wait_until_connected(self, uid):
         delay = 0.05
@@ -569,7 +537,6 @@ class WorkerProcess(object):
             """
             Try to start worker and inform parent of outcome.
             """
-            init_result_q.put({'uid': uid, 'status': 'started'})
             try:
                 yield worker._start(*worker_start_args)
             except Exception as e:
